@@ -20,7 +20,6 @@ import mock
 import six
 
 from novaclient import api_versions
-from novaclient import base
 from novaclient import exceptions
 from novaclient.tests.unit.fixture_data import client
 from novaclient.tests.unit.fixture_data import floatingips
@@ -30,21 +29,6 @@ from novaclient.tests.unit.v2 import fakes
 from novaclient.v2 import servers
 
 
-class _FloatingIPManager(base.Manager):
-    resource_class = base.Resource
-
-    @api_versions.deprecated_after('2.35')
-    def list(self):
-        """DEPRECATED: List floating IPs"""
-        return self._list("/os-floating-ips", "floating_ips")
-
-    @api_versions.deprecated_after('2.35')
-    def get(self, floating_ip):
-        """DEPRECATED: Retrieve a floating IP"""
-        return self._get("/os-floating-ips/%s" % base.getid(floating_ip),
-                         "floating_ip")
-
-
 class ServersTest(utils.FixturedTestCase):
 
     client_fixture_class = client.V1
@@ -56,7 +40,6 @@ class ServersTest(utils.FixturedTestCase):
         self.useFixture(floatingips.FloatingFixture(self.requests_mock))
         if self.api_version:
             self.cs.api_version = api_versions.APIVersion(self.api_version)
-        self.floating_ips = _FloatingIPManager(self.cs)
 
     def _get_server_create_default_nics(self):
         """Callback for default nics kwarg when creating a server.
@@ -188,7 +171,7 @@ class ServersTest(utils.FixturedTestCase):
                 nics=nics
             )
             self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
-            self.assert_called('POST', '/os-volumes_boot')
+            self.assert_called('POST', '/servers')
             self.assertIsInstance(s, servers.Server)
 
         test_create_server_from_volume()
@@ -217,13 +200,13 @@ class ServersTest(utils.FixturedTestCase):
                 nics=self._get_server_create_default_nics()
             )
             self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
-            self.assert_called('POST', '/os-volumes_boot')
+            self.assert_called('POST', '/servers')
             self.assertIsInstance(s, servers.Server)
 
     def test_create_server_boot_with_nics_ipv6(self):
         old_boot = self.cs.servers._boot
         nics = [{'net-id': '11111111-1111-1111-1111-111111111111',
-                'v6-fixed-ip': '2001:db9:0:1::10'}]
+                 'v6-fixed-ip': '2001:db9:0:1::10'}]
 
         def wrapped_boot(url, key, *boot_args, **boot_kwargs):
             self.assertEqual(boot_kwargs['nics'], nics)
@@ -390,6 +373,17 @@ class ServersTest(utils.FixturedTestCase):
     def test_create_server_disk_config_manual(self):
         self._create_disk_config('MANUAL')
 
+    def test_create_server_return_reservation_id(self):
+        s = self.cs.servers.create(
+            name="My server",
+            image=1,
+            flavor=1,
+            reservation_id=True,
+            nics=self._get_server_create_default_nics()
+        )
+        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers')
+
     def test_update_server(self):
         s = self.cs.servers.get(1234)
 
@@ -556,81 +550,6 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-    @mock.patch('warnings.warn')
-    def test_add_fixed_ip(self, mock_warn):
-        s = self.cs.servers.get(1234)
-        fip = s.add_fixed_ip(1)
-        mock_warn.assert_called_once()
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        fip = self.cs.servers.add_fixed_ip(s, 1)
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-
-    @mock.patch('warnings.warn')
-    def test_remove_fixed_ip(self, mock_warn):
-        s = self.cs.servers.get(1234)
-        ret = s.remove_fixed_ip('10.0.0.1')
-        mock_warn.assert_called_once()
-        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        ret = self.cs.servers.remove_fixed_ip(s, '10.0.0.1')
-        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-
-    @mock.patch('warnings.warn')
-    def test_add_floating_ip(self, mock_warn):
-        s = self.cs.servers.get(1234)
-        fip = s.add_floating_ip('11.0.0.1')
-        mock_warn.assert_called_once()
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        fip = self.cs.servers.add_floating_ip(s, '11.0.0.1')
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        f = self.floating_ips.list()[0]
-        fip = self.cs.servers.add_floating_ip(s, f)
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        fip = s.add_floating_ip(f)
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-
-    def test_add_floating_ip_to_fixed(self):
-        s = self.cs.servers.get(1234)
-        fip = s.add_floating_ip('11.0.0.1', fixed_address='12.0.0.1')
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        fip = self.cs.servers.add_floating_ip(s, '11.0.0.1',
-                                              fixed_address='12.0.0.1')
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        f = self.floating_ips.list()[0]
-        fip = self.cs.servers.add_floating_ip(s, f)
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        fip = s.add_floating_ip(f)
-        self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-
-    @mock.patch('warnings.warn')
-    def test_remove_floating_ip(self, mock_warn):
-        s = self.cs.servers.get(1234)
-        ret = s.remove_floating_ip('11.0.0.1')
-        mock_warn.assert_called_once()
-        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        ret = self.cs.servers.remove_floating_ip(s, '11.0.0.1')
-        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        f = self.floating_ips.list()[0]
-        ret = self.cs.servers.remove_floating_ip(s, f)
-        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-        ret = s.remove_floating_ip(f)
-        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/servers/1234/action')
-
     def test_stop(self):
         s = self.cs.servers.get(1234)
         ret = s.stop()
@@ -963,7 +882,7 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
         ret = self.cs.servers.evacuate(s, 'fake_target_host',
-                                          'False', 'NewAdminPassword')
+                                       'False', 'NewAdminPassword')
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
@@ -1034,6 +953,18 @@ class ServersTest(utils.FixturedTestCase):
                           flavor='1',
                           nics='auto')
 
+    def test__validate_create_nics(self):
+        if self.cs.api_version > api_versions.APIVersion('2.36'):
+            self.assertRaises(ValueError,
+                              self.cs.servers._validate_create_nics, None)
+        else:
+            self.cs.servers._validate_create_nics(None)
+            self.assertRaises(ValueError,
+                              self.cs.servers._validate_create_nics,
+                              mock.Mock())
+        self.cs.servers._validate_create_nics(["foo", "bar"])
+        self.cs.servers._validate_create_nics(("foo", "bar"))
+
 
 class ServersV26Test(ServersTest):
 
@@ -1361,7 +1292,7 @@ class ServersV232Test(ServersV226Test):
                                    key_name="fakekey",
                                    block_device_mapping_v2=bdm)
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('POST', '/os-volumes_boot')
+        self.assert_called('POST', '/servers')
 
     def test_create_server_boot_from_volume_tagged_bdm_v2_pre232(self):
         self.cs.api_version = api_versions.APIVersion("2.31")
@@ -1398,18 +1329,6 @@ class ServersV2_37Test(ServersV226Test):
         self.assert_called('POST', '/servers')
         self.assertIsInstance(s, servers.Server)
 
-    def test_add_floating_ip(self):
-        # self.floating_ips.list() is not available after 2.35
-        pass
-
-    def test_add_floating_ip_to_fixed(self):
-        # self.floating_ips.list() is not available after 2.35
-        pass
-
-    def test_remove_floating_ip(self):
-        # self.floating_ips.list() is not available after 2.35
-        pass
-
 
 class ServersCreateImageBackupV2_45Test(utils.FixturedTestCase):
     """Tests the 2.45 microversion for createImage and createBackup
@@ -1527,3 +1446,57 @@ class ServersV252Test(ServersV249Test):
                           key_name="fakekey",
                           nics=self._get_server_create_default_nics(),
                           tags=['tag1', 'tag2'])
+
+
+class ServersV254Test(ServersV252Test):
+
+    api_version = "2.54"
+
+    def test_rebuild_with_key_name(self):
+        s = self.cs.servers.get(1234)
+        ret = s.rebuild(image="1", key_name="test_keypair")
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'rebuild': {
+                               'imageRef': '1',
+                               'key_name': 'test_keypair'}})
+
+    def test_rebuild_with_key_name_none(self):
+        s = self.cs.servers.get(1234)
+        ret = s.rebuild(image="1", key_name=None)
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'rebuild': {
+                               'key_name': None,
+                               'imageRef': '1'}})
+
+    def test_rebuild_with_key_name_pre_254_fails(self):
+        self.cs.api_version = api_versions.APIVersion('2.53')
+        ex = self.assertRaises(exceptions.UnsupportedAttribute,
+                               self.cs.servers.rebuild,
+                               '1234', fakes.FAKE_IMAGE_UUID_1,
+                               key_name='test_keypair')
+        self.assertIn('key_name', six.text_type(ex.message))
+
+
+class ServersV256Test(ServersV254Test):
+
+    api_version = "2.56"
+
+    def test_migrate_server(self):
+        s = self.cs.servers.get(1234)
+        ret = s.migrate()
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'migrate': {}})
+        ret = s.migrate(host='target-host')
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'migrate': {'host': 'target-host'}})
+
+    def test_migrate_server_pre_256_fails(self):
+        self.cs.api_version = api_versions.APIVersion('2.55')
+        s = self.cs.servers.get(1234)
+        ex = self.assertRaises(TypeError,
+                               s.migrate, host='target-host')
+        self.assertIn('host', six.text_type(ex))
